/*
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include "Pickers/DrumChoice.h"
#include "GUI/GraphicalTrack.h"
#include "Midi/Sequence.h"
#include "GUI/GLPane.h"
#include "IO/IOUtils.h"
#include <iostream>

#include "Config.h"

namespace AriaMaestosa {
	
BEGIN_EVENT_TABLE(DrumChoice, wxMenu)

EVT_MENU_RANGE(0,127,DrumChoice::menuSelected)

END_EVENT_TABLE()


DrumChoice::DrumChoice(GraphicalTrack* parent) : wxMenu()
{
    INIT_LEAK_CHECK();
	
    drumID=0;
    
    Append( 0 ,wxT("Standard"));
    Append( 8 ,wxT("Room kit"));
    Append( 16 ,wxT("Power kit"));
    Append( 24 ,wxT("Electronic"));
    Append( 25 ,wxT("Analog"));
    Append( 32 ,wxT("Jazz"));
    Append( 40 ,wxT("Brush"));
    Append( 48 ,wxT("Orchestral"));
    Append( 56 ,wxT("Special Effects"));
    
    DrumChoice::parent = parent;
}

DrumChoice::~DrumChoice()
{
}

void DrumChoice::menuSelected(wxCommandEvent& evt)
{
    
    drumID=evt.GetId();
    

    assertExpr(drumID,<,128);
    assertExpr(drumID,>=,0);
    
    parent->glPane->render();
}

void DrumChoice::setDrumID(int id)
{
    drumID = id;
}

int DrumChoice::getDrumID()
{
    return drumID;   
}

char* DrumChoice::getDrumName()
{
    if (drumID == 0 ) return "Standard";
    else if (drumID == 8 ) return "Room kit";
    else if (drumID == 16 ) return "Power kit";
    else if (drumID == 24 ) return "Electronic";
    else if (drumID == 25 ) return "Analog";
    else if (drumID == 32 ) return "Jazz";
    else if (drumID == 40 ) return "Brush";
    else if (drumID == 48 ) return "Orchestral";
    else if (drumID == 56 ) return "Special Effects";
    else
	{
        wxBell();
        std::cout << "wrong drumset ID: " << drumID << std::endl;
        drumID = 0;
        return "Wrong drumset ID";
    }
}

void DrumChoice::saveToFile(wxFileOutputStream& fileout)
{
	writeData( wxString("<drumkit id=\"") + to_wxString(drumID) + wxString("\"/>\n"), fileout);
}

bool DrumChoice::readFromFile(irr::io::IrrXMLReader* xml)
{
	
	const char* id = xml->getAttributeValue("id");
	if(id != NULL) drumID = atoi(id);
	else
	{
		std::cout << "Missing info from file: drum ID" << std::endl;
		drumID = 0;
	}

	return true;
}

}
